/*
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: t-mtparse-0.c,v 1.5 2008/04/22 04:16:01 ca Exp $")
#include "sm/error.h"
#include "sm/assert.h"
#include "sm/ctype.h"
#include "sm/io.h"
#include "sm/net.h"
#include "sm/misc.h"
#include "sm/sysexits.h"
#include "sm/test.h"
#include "smar.h"
#define SMAR_LOG_DEFINES 1
#include "log.h"
#include "mtparse.h"
#include "t-init.h"

int Loglevel = 0;
static int Verbose = 0;

static void
prtcnf(ar_mtcnf_P ar_mtcnf)
{
	if (Verbose) {
		sm_io_fprintf(smioerr,
			"protocol=%d\n"
			"port=%d\n"
			"host=%s\n"
			"ipv4[0]=%s\n"
			"n_ip=%u\n"
			"ipv4[0]=%A\n"
			, ar_mtcnf->ar_mt_cnf_prot
			, ar_mtcnf->ar_mt_cnf_port
			, ar_mtcnf->ar_mt_cnf_host
			, ar_mtcnf->ar_mt_cnf_ips[0]
			, ar_mtcnf->ar_mt_cnf_n_ipv4s
			, ar_mtcnf->ar_mt_cnf_ipv4s[0]
			);
	}
}

static sm_ret_T
test_mtparse(smar_ctx_T smar_ctx, smar_clt_ctx_P smar_clt_ctx, ar_mtcnf_P ar_mtcnf, sm_str_P mtrhs)
{
	sm_ret_T ret;

	sm_str_clr(mtrhs);
	ret = sm_str_scat0(mtrhs, "host=meta1.org;");
	SM_TEST_E(ret == SM_SUCCESS);
	sm_memzero(ar_mtcnf, sizeof(*ar_mtcnf));
	ret = smar_mt_parse(smar_clt_ctx, mtrhs, ar_mtcnf);
	SM_TEST_E(ret == SM_SUCCESS);
	SM_TEST(0 == ar_mtcnf->ar_mt_cnf_prot);
	SM_TEST(0 == ar_mtcnf->ar_mt_cnf_port);
	SM_TEST(strcmp(ar_mtcnf->ar_mt_cnf_host,"meta1.org") == 0);
	SM_TEST(NULL == ar_mtcnf->ar_mt_cnf_ips[0]);
	SM_TEST(0 == ar_mtcnf->ar_mt_cnf_n_ipv4s);
	SM_TEST(0 == ar_mtcnf->ar_mt_cnf_ipv4s[0]);
	prtcnf(ar_mtcnf);
	if (smar_clt_ctx->smac_cnf != NULL) {
		sm_conf_destroy(smar_clt_ctx->smac_cnf);
		smar_clt_ctx->smac_cnf = NULL;
	}

	sm_str_clr(mtrhs);
	ret = sm_str_scat0(mtrhs, "protocol=esmtp; host=meta1.org;");
	SM_TEST_E(ret == SM_SUCCESS);
	sm_memzero(ar_mtcnf, sizeof(*ar_mtcnf));
	ret = smar_mt_parse(smar_clt_ctx, mtrhs, ar_mtcnf);
	SM_TEST_E(ret == SM_SUCCESS);
	SM_TEST(SMAR_PROT_ESMTP == ar_mtcnf->ar_mt_cnf_prot);
	SM_TEST(0 == ar_mtcnf->ar_mt_cnf_port);
	SM_TEST(strcmp(ar_mtcnf->ar_mt_cnf_host,"meta1.org") == 0);
	SM_TEST(NULL == ar_mtcnf->ar_mt_cnf_ips[0]);
	SM_TEST(0 == ar_mtcnf->ar_mt_cnf_n_ipv4s);
	SM_TEST(0 == ar_mtcnf->ar_mt_cnf_ipv4s[0]);
	prtcnf(ar_mtcnf);
	if (smar_clt_ctx->smac_cnf != NULL) {
		sm_conf_destroy(smar_clt_ctx->smac_cnf);
		smar_clt_ctx->smac_cnf = NULL;
	}

	sm_str_clr(mtrhs);
	ret = sm_str_scat0(mtrhs, "protocol=lmtp; port=6789; ipv4={1.2.3.4, 10.11.12.123};");
	SM_TEST_E(ret == SM_SUCCESS);
	sm_memzero(ar_mtcnf, sizeof(*ar_mtcnf));
	ret = smar_mt_parse(smar_clt_ctx, mtrhs, ar_mtcnf);
	SM_TEST_E(ret == SM_SUCCESS);
	SM_TEST(SMAR_PROT_LMTP == ar_mtcnf->ar_mt_cnf_prot);
	SM_TEST(6789 == ar_mtcnf->ar_mt_cnf_port);
	SM_TEST(NULL == ar_mtcnf->ar_mt_cnf_host);
	SM_TEST(NULL != ar_mtcnf->ar_mt_cnf_ips[0]);
	SM_TEST(2 == ar_mtcnf->ar_mt_cnf_n_ipv4s);
	SM_TEST(inet_addr("1.2.3.4") == ar_mtcnf->ar_mt_cnf_ipv4s[0]);
	SM_TEST(inet_addr("10.11.12.123") == ar_mtcnf->ar_mt_cnf_ipv4s[1]);
	prtcnf(ar_mtcnf);
	if (smar_clt_ctx->smac_cnf != NULL) {
		sm_conf_destroy(smar_clt_ctx->smac_cnf);
		smar_clt_ctx->smac_cnf = NULL;
	}

	sm_str_clr(mtrhs);
	ret = sm_str_scat0(mtrhs, "protocol=lmtp; ipv4={1.2.3.4, 10.11.12.123};");
	SM_TEST_E(ret == SM_SUCCESS);
	sm_memzero(ar_mtcnf, sizeof(*ar_mtcnf));
	ret = smar_mt_parse(smar_clt_ctx, mtrhs, ar_mtcnf);
	SM_TEST_E(ret != SM_SUCCESS);
	SM_TEST(SMAR_PROT_LMTP == ar_mtcnf->ar_mt_cnf_prot);
	SM_TEST(NULL == ar_mtcnf->ar_mt_cnf_host);
	SM_TEST(NULL != ar_mtcnf->ar_mt_cnf_ips[0]);
	SM_TEST(2 == ar_mtcnf->ar_mt_cnf_n_ipv4s);
	SM_TEST(inet_addr("1.2.3.4") == ar_mtcnf->ar_mt_cnf_ipv4s[0]);
	SM_TEST(inet_addr("10.11.12.123") == ar_mtcnf->ar_mt_cnf_ipv4s[1]);
	prtcnf(ar_mtcnf);
	if (smar_clt_ctx->smac_cnf != NULL) {
		sm_conf_destroy(smar_clt_ctx->smac_cnf);
		smar_clt_ctx->smac_cnf = NULL;
	}

	sm_str_clr(mtrhs);
	ret = sm_str_scat0(mtrhs, "host=meta1.org; ipv4={1.2.3.4};");
	SM_TEST_E(ret == SM_SUCCESS);
	sm_memzero(ar_mtcnf, sizeof(*ar_mtcnf));
	ret = smar_mt_parse(smar_clt_ctx, mtrhs, ar_mtcnf);
	SM_TEST_E(ret != SM_SUCCESS);
	SM_TEST(0 == ar_mtcnf->ar_mt_cnf_prot);
	SM_TEST(NULL != ar_mtcnf->ar_mt_cnf_host);
	SM_TEST(NULL != ar_mtcnf->ar_mt_cnf_ips[0]);
	SM_TEST(1 == ar_mtcnf->ar_mt_cnf_n_ipv4s);
	SM_TEST(inet_addr("1.2.3.4") == ar_mtcnf->ar_mt_cnf_ipv4s[0]);
	prtcnf(ar_mtcnf);
	if (smar_clt_ctx->smac_cnf != NULL) {
		sm_conf_destroy(smar_clt_ctx->smac_cnf);
		smar_clt_ctx->smac_cnf = NULL;
	}

	sm_str_clr(mtrhs);
	ret = sm_str_scat0(mtrhs, "protocol=smtp; ipv4={1.2.3.4};");
	SM_TEST_E(ret == SM_SUCCESS);
	sm_memzero(ar_mtcnf, sizeof(*ar_mtcnf));
	ret = smar_mt_parse(smar_clt_ctx, mtrhs, ar_mtcnf);
	SM_TEST_E(ret != SM_SUCCESS);
	SM_TEST(SMAR_PROT_LMTP != ar_mtcnf->ar_mt_cnf_prot);
	prtcnf(ar_mtcnf);
	if (smar_clt_ctx->smac_cnf != NULL) {
		sm_conf_destroy(smar_clt_ctx->smac_cnf);
		smar_clt_ctx->smac_cnf = NULL;
	}

	sm_str_clr(mtrhs);
	ret = sm_str_scat0(mtrhs, "ipv4={1.2.3.};");
	SM_TEST_E(ret == SM_SUCCESS);
	sm_memzero(ar_mtcnf, sizeof(*ar_mtcnf));
	ret = smar_mt_parse(smar_clt_ctx, mtrhs, ar_mtcnf);
	SM_TEST_E(ret != SM_SUCCESS);
	SM_TEST(SMAR_PROT_LMTP != ar_mtcnf->ar_mt_cnf_prot);
	prtcnf(ar_mtcnf);
	if (smar_clt_ctx->smac_cnf != NULL) {
		sm_conf_destroy(smar_clt_ctx->smac_cnf);
		smar_clt_ctx->smac_cnf = NULL;
	}

  error:
	return ret;
}

int
main(int argc, char *argv[])
{
	int c, i;
	sm_ret_T ret;
	sm_str_P mtrhs;
	ar_mtcnf_T ar_mtcnf;
	smar_clt_ctx_P smar_clt_ctx;
	smar_ctx_T smar_ctx;

	sm_test_begin(argc, argv, "test mtparse 0");
/*
	argc -= optind;
	argv += optind;
*/

	while ((c = getopt(argc, argv, "V")) != -1)
	{
		switch (c)
		{
		  case 'V':
			++Verbose;
		  default:
			return EX_USAGE;
		}
	}

	ret = t_smar_0_init(&smar_ctx, 0);
	SM_TEST_E(ret == SM_SUCCESS);
	ret = smar_clt_new(&smar_ctx, &smar_clt_ctx);
	SM_TEST_E(ret == SM_SUCCESS);

	mtrhs = sm_str_new(NULL, 16, 256);
	SM_TEST_E(mtrhs != NULL);

	if (argc < 2) {
		ret = test_mtparse(smar_ctx, smar_clt_ctx, &ar_mtcnf, mtrhs);
	}
	else {
		for (i = 1; i < argc; i++) {
			sm_str_clr(mtrhs);
			ret = sm_str_scat0(mtrhs, argv[i]);
			SM_TEST_E(ret == SM_SUCCESS);
sm_io_fprintf(smioerr, "parse %@S\n", mtrhs);
			sm_memzero(&ar_mtcnf, sizeof(ar_mtcnf));
			ret = smar_mt_parse(smar_clt_ctx, mtrhs, &ar_mtcnf);
			SM_TEST_E(ret == SM_SUCCESS);
sm_io_fprintf(smioerr, "parsed=%r\n", ret);
sm_io_fprintf(smioerr,
	"protocol=%d\n"
	"port=%d\n"
	"host=%s\n"
	"ipv4[0]=%s\n"
	"n_ip=%u\n"
	"ipv4[0]=%A\n"
	, ar_mtcnf.ar_mt_cnf_prot
	, ar_mtcnf.ar_mt_cnf_port
	, ar_mtcnf.ar_mt_cnf_host
	, ar_mtcnf.ar_mt_cnf_ips[0]
	, ar_mtcnf.ar_mt_cnf_n_ipv4s
	, ar_mtcnf.ar_mt_cnf_ipv4s[0]
	);
		}
	}
 error:
	return sm_test_end();
}
